#include "demo.h"
#include "ui_demo.h"

#include <QApplication>

#include <QGraphicsRectItem>
#include <QGraphicsEllipseItem>

#include <QFileDialog>
#include <QClipboard>

#include <QXmlStreamReader>
#include <QXmlStreamWriter>

#include "colorbutton.h"
#include "toolbutton.h"
#include "drop.h"
#include "io.h"
#include "property.h"
#include "tree.h"
#include "view.h"

MainWindow::MainWindow (QWidget * parent) :
    QMainWindow (parent),
    ui (new Ui::MainWindow)
{
    ui->setupUi(this);

    ui->hsplitter->setStretchFactor(0, 1);
    ui->hsplitter->setStretchFactor(1, 4);
    ui->hsplitter->setStretchFactor(2, 2);

    ui->vsplitter->setStretchFactor(0, 3);
    ui->vsplitter->setStretchFactor(1, 1);

    ui->palette->removeTab (0);

    QToolBar * toolPage = new QToolBar;
    addToolButtons (toolPage);
    ui->palette->addTab (toolPage, "Tools");
    ui->palette->setCurrentWidget(toolPage);

    QToolBar * colorPage = new QToolBar;
    addColorButtons (colorPage);
    ui->palette->addTab (colorPage, "Colors");
    // ui->palette->setCurrentWidget (colorPage);

    ui->statusbar->showMessage (QString ("Qt ") + qVersion());

    // scene = new QGraphicsScene;
    scene = new Scene;
    ui->graphicsView->setScene (scene);
    scene->setSceneRect(0, 0, 800, 600);
    // scene->addLine (0, 0, 100, 200, QColor ("red"));

    scene->setWin (this);

    ui->tree->setScene (scene);
    ui->tree->setWin (this);

    ui->prop->setWin (this);

    openFile (":/data/abc.xml");
}

MainWindow::~MainWindow()
{
    delete ui;
}

/* open / save */

void MainWindow::on_actionOpen_triggered ()
{
    QString fileName = QFileDialog::getOpenFileName (this, "Open file");
    if (fileName != "")
       openFile (fileName);
}

void MainWindow::openFile (QString fileName)
{
    QFile f (fileName);
    if (f.open (QFile::ReadOnly))
    {
        QXmlStreamReader r (&f);
        readXml (r, scene);
    }
    refreshTree ();
}

void MainWindow::on_actionSave_triggered ()
{
    QString fileName = QFileDialog::getSaveFileName (this, "Save file");
    if (fileName != "")
       saveFile (fileName);
}

void MainWindow::saveFile (QString fileName)
{
    QFile f (fileName);
    if (f.open (QFile::WriteOnly))
    {
        QXmlStreamWriter w (&f);
        writeXml (w, scene);
    }
}

/* clipboard */

void MainWindow::on_actionCopy_triggered ()
{
    QString code = "";
    QXmlStreamWriter writer (& code);
    writeBegin (writer);

    for (QGraphicsItem * item : scene->selectedItems ())
    {
        writeItem (writer, item);
    }

    writeEnd (writer);

    QMimeData * data = new QMimeData;
    data->setData (shapeFormat, code.toLatin1 ());

    QClipboard * clip = QApplication::clipboard ();
    clip->setMimeData (data);
}

void MainWindow::on_actionPaste_triggered ()
{
    QClipboard * clip = QApplication::clipboard ();
    const QMimeData * data = clip->mimeData ();
    if (data->hasFormat (shapeFormat))
    {
       QString code = data->data (shapeFormat);
       QXmlStreamReader reader (code);
       readXml (reader, scene);
    }
    refreshTree ();
}

/* tree */

void MainWindow::refreshTree ()
{
    ui->tree->displayTree ();
}

void MainWindow::refreshProperties (QGraphicsItem * item)
{
    ui->prop->displayProperties (item);
}

void MainWindow::refreshTreeName (QGraphicsItem * item, QString name)
{
    ui->tree->renameTreeItem (item, name);
}

void MainWindow::on_tree_itemDoubleClicked (QTreeWidgetItem * node, int column)
{
    TreeNode * tree_node = dynamic_cast <TreeNode *> (node);
    if (tree_node != nullptr && tree_node->item != nullptr)
       refreshProperties (tree_node->item);
    else
       refreshProperties (nullptr);
}

void MainWindow::on_tree_itemSelectionChanged ()
{
    QGraphicsItem * item = nullptr;
    QList <QTreeWidgetItem * > list = ui->tree->selectedItems ();
    if (list.count () == 1)
    {
       TreeNode * node = dynamic_cast < TreeNode * > (list[0]);
       if (node != nullptr)
          item = node->item;
    }
    refreshProperties (item);
}

void MainWindow::on_actionRun_triggered ()
{
   QGraphicsRectItem * block = new QGraphicsRectItem;
   block->setRect (0, 0, 200, 160);
   block->setPos (100, 100);
   block->setPen (QColor ("orange"));
   block->setBrush (QColor ("yellow"));
   block->setToolTip ("block");
   block->setFlags (QGraphicsItem::ItemIsMovable | QGraphicsEllipseItem::ItemIsSelectable);
   scene->addItem (block);

   for (int i = 1; i <= 2; i++)
   {
       QGraphicsEllipseItem * item = new QGraphicsEllipseItem;
       item->setRect (0, 0, 40, 40);
       item->setPos (40 + (i-1) * 80, 40);
       item->setPen (QColor ("blue"));
       item->setBrush (QColor ("cornflowerblue"));
       item->setToolTip ("item" + QString::number (i));
       item->setFlags (QGraphicsItem::ItemIsMovable | QGraphicsEllipseItem::ItemIsSelectable);
       item->setParentItem (block);
   }

   refreshTree ();
}

void MainWindow::on_actionQuit_triggered ()
{
   close ();
}

int main (int argc, char * argv [])
{
    QApplication a (argc, argv);
    MainWindow w;
    w.show ();
    return a.exec ();
}

